#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <malloc.h>
#include <ogcsys.h>
#include <gccore.h>
#include <ogc/es.h>
#include <sys/errno.h>
#include <ogc/isfs.h>
#include <ogc/ipc.h>
#include <ogc/ios.h>
#include <wiiuse/wpad.h>
#include <ogc/wiilaunch.h>
#include <fcntl.h>
#include <sys/unistd.h>
#include <sys/fcntl.h>
#include <fat.h>
#include <dirent.h>
#include <sdcard/wiisd_io.h>
#include <usbstorage.h>

#include "id.h"
#include "sha1.h"
#include "usbstorage.h"

#define MINIMAL_DUMP 0
#define COMPATIBILITY_TEST_DUMP 1
#define FULL_DUMP 2

#define BLOCKSIZE 2048

#define DIRENT_T_FILE 0
#define DIRENT_T_DIR 1

#define INIT_FIRST 1
#define INIT_RESET 0

u32 fs_size = 0;

static char *device;


//Types.

typedef struct _dirent
{
	char name[ISFS_MAXPATH + 1];
	int type;
	u32 ownerID;
	u16 groupID;
	u8 attributes;
	u8 ownerperm;
	u8 groupperm;
	u8 otherperm;
} dirent_t;

typedef struct _dir
{
	char name[ISFS_MAXPATH + 1];
} dir_t;

typedef struct _list
{
	char name[ISFS_MAXPATH + 1];

} list_t;

/* Video pointers */
static GXRModeObj *rmode = NULL;
u32 *xfb;
bool Power_Flag;
bool Reset_Flag;

void reset_log()
{
	char buf[256];
	sprintf(buf, "%ssimplefsdumper.log", device);
	remove(buf);
}	

void logfile(const char *format, ...)
{
	char buffer[256];
	va_list args;
	va_start (args, format);
	vsprintf (buffer,format, args);
	FILE *f;
	char buf[26];
	sprintf(buf, "%ssimplefsdumper.log", device);
	f = fopen(buf, "a");
	if (!f)
	{
		printf("Error writing log\n");
		sleep(7);
	    exit(0);
		return;
	}
	fputs(buffer, f);
	fclose(f);
	va_end (args);
}

void hexdump_log(void *d, int len)
{
    u8 *data;
    int i, f, off;
    data = (u8*)d;
    for (off=0; off<len; off += 16) 
	{
        logfile("%08x:  ",16*(off/16));
		for(f=0; f < 16; f += 4)
		{
			for(i=0; i<4; i++)
			{
				if((i+off)>=len)
				{
					logfile(" ");
				} else
				{
					logfile("%02x",data[off+f+i]);
				}  
                
				//logfile(" ");
			}
			logfile(" ");
		}	
		logfile("\n");
		//logfile(" ");
    }		
    logfile("\n");
}

// Prevent IOS36 loading at startup
s32 __IOS_LoadStartupIOS()
{
	return 0;
}

static void power_cb () 
{
	Power_Flag = true;
}

static void reset_cb () 
{
	Reset_Flag = true;
}

void Verify_Flags()
{
	if (Power_Flag)
	{
		WPAD_Shutdown();
		STM_ShutdownToStandby();
	}
	if (Reset_Flag)
	{
		STM_RebootSystem();
	}
}

void Reboot()
{
	logfile("Exit...\n");
	if (*(u32*)0x80001800) exit(0);
	logfile("Reset...\n");
	SYS_ResetSystem(SYS_RETURNTOMENU, 0, 0);
}

void waitforbuttonpress(u32 *out, u32 *outGC)
{
	u32 pressed = 0;
	u32 pressedGC = 0;

	while (true)
	{
		WPAD_ScanPads();
		pressed = WPAD_ButtonsDown(0);

		PAD_ScanPads();
		pressedGC = PAD_ButtonsDown(0);

		if(pressed || pressedGC) 
		{
			if (pressedGC)
			{
				// Without waiting you can't select anything
				usleep (20000);
			}
			if (out) *out = pressed;
			if (outGC) *outGC = pressedGC;
			return;
		}
	}
}

void Con_ClearLine()
{
	s32 cols, rows;
	u32 cnt;

	printf("\r");
	fflush(stdout);

	/* Get console metrics */
	CON_GetMetrics(&cols, &rows);

	/* Erase line */
	for (cnt = 1; cnt < cols; cnt++) {
		printf(" ");
		fflush(stdout);
	}

	printf("\r");
	fflush(stdout);
}

static void sys_init(void)
{
	// Initialise the video system
	VIDEO_Init();
	
	// Obtain the preferred video mode from the system
	// This will correspond to the settings in the Wii menu
	rmode = VIDEO_GetPreferredMode(NULL);

	// Allocate memory for the display in the uncached region
	xfb = MEM_K0_TO_K1(SYS_AllocateFramebuffer(rmode));
	
	// Set up the video registers with the chosen mode
	VIDEO_Configure(rmode);
	
	// Tell the video hardware where our display memory is
	VIDEO_SetNextFramebuffer(xfb);
	
	// Make the display visible
	VIDEO_SetBlack(FALSE);

	// Flush the video register changes to the hardware
	VIDEO_Flush();

	// Wait for Video setup to complete
	VIDEO_WaitVSync();
	if(rmode->viTVMode&VI_NON_INTERLACE) VIDEO_WaitVSync();

	// Set console parameters
    int x = 24, y = 32, w, h;
    w = rmode->fbWidth - (32);
    h = rmode->xfbHeight - (48);

    // Initialize the console - CON_InitEx works after VIDEO_ calls
	CON_InitEx(rmode, x, y, w, h);

	// Clear the garbage around the edges of the console
    VIDEO_ClearFrameBuffer(rmode, xfb, COLOR_BLACK);
}

void resetscreen()
{
	printf("\x1b[2J");
}

s32 ignore_folder(char *path)
{
	s32 ret;
		
	ret = strcmp(path, "/sys/");
	if(ret == 0)
		return 0;
		
	ret = strcmp(path, "/import");
	if(ret == 0)
		return 0;	
		
	ret = strcmp(path, "/shared1");
	if(ret == 0)
		return 0;	
	
	ret = strcmp(path, "/sys");
	if(ret == 0)
		return 0;	
		
	ret = strcmp(path, "/ticket");
	if(ret == 0)
		return 0;	
		
	ret = strcmp(path, "/title");
	if(ret == 0)
		return 0;	
		
	ret = strcmp(path, "/title/00000001");
	if(ret == 0)
		return 0;	
		
	ret = strcmp(path, "/tmp");
	if(ret == 0)
		return 0;	
		
	ret = strcmp(path, "/wfs");
	if(ret == 0)
		return 0;	
		
	return 1;	
	
}	
s32 ignore_folder_test(char *path)
{
	s32 ret;
	if(strstr(path, "/title/00000001/") != 0)
	{
		
		if(strstr(path, "/title/00000001/00000002") != 0)
		{
			return 1;
		} else
		{
			return 0;
		}	
		
		//return 0;
	
	}	
	
	ret = strcmp(path, "/title/00000001");
	if(ret == 0)
		return 0;	
	
	ret = strcmp(path, "/title/00010000");
	if(ret == 0)
		return 0;

	ret = strcmp(path, "/title/00010002");
	if(ret == 0)
		return 0;

	ret = strcmp(path, "/title/00010004");
	if(ret == 0)
		return 0;
		
	ret = strcmp(path, "/title/00010006");
	if(ret == 0)
		return 0;
		
	ret = strcmp(path, "/title/00010008");
	if(ret == 0)
		return 0;
		
	ret = strcmp(path, "/ticket/00000001");
	if(ret == 0)	
		return 0;		
	
	ret = strcmp(path, "/ticket/00010000");
	if(ret == 0)
		return 0;

	ret = strcmp(path, "/ticket/00010002");
	if(ret == 0)
		return 0;

	ret = strcmp(path, "/ticket/00010005");
	if(ret == 0)
		return 0;
		
	ret = strcmp(path, "/ticket/00010008");
	if(ret == 0)
		return 0;	
		
	ret = strcmp(path, "/title/00010000");
	if(ret == 0)
		return 0;

	ret = strcmp(path, "/title/00010002");
	if(ret == 0)
		return 0;

	ret = strcmp(path, "/title/00010004");
	if(ret == 0)
		return 0;
		
	ret = strcmp(path, "/title/00010006");
	if(ret == 0)
		return 0;
		
	ret = strcmp(path, "/title/00010008");
	if(ret == 0)
		return 0;	
	
	ret = strcmp(path, "/ticket/00010000");
	if(ret == 0)
		return 0;

	ret = strcmp(path, "/ticket/00010002");
	if(ret == 0)
		return 0;

	ret = strcmp(path, "/ticket/00010005");
	if(ret == 0)
		return 0;
		
	ret = strcmp(path, "/ticket/00010008");
	if(ret == 0)
		return 0;
		
	return 1;	
	
}	


s32 dumpfile(char source[1024], char destination[1024])
{
	Verify_Flags();
	int buttonsdown = 0;
	int buttonsdownGC = 0;
	
	WPAD_ScanPads();
	buttonsdown = WPAD_ButtonsDown(0);
	PAD_ScanPads();
	buttonsdownGC = PAD_ButtonsDown(0);

	if ((buttonsdown & WPAD_BUTTON_B) || (buttonsdownGC & PAD_BUTTON_B)) 
	{
		printf("\nB button pressed...\n");
		return -1;
	}
	
	char *hmm = NULL;
	int place = 0;
	hmm = strstr(destination, "HP:");

	if(hmm != NULL)
	{
		logfile("Found character that is not supported on FAT, changing ':' to '_'\n");
		place = strlen(destination)-strlen(hmm);
		logfile("place = %d\n", place+1);
		logfile("strncpy(hmm, 'HP_', 3);\n");
		strncpy(hmm, "HP_", 3);
		logfile("new destination = %s\n", destination);
	}	
	
	u8 *buffer;
	fstats *status;

	FILE *file;
	int fd;
	s32 ret;
	u32 size;

	fd = ISFS_Open(source, ISFS_OPEN_READ);
	if (fd < 0) 
	{
	
		return fd;
	}
	
	file = fopen(destination, "wb+");
	if (!file)
	{
		printf("\nError: fopen(%s) returned 0\n", destination);
		logfile("\nError: fopen(%s) returned 0\n", destination);
		ISFS_Close(fd);
		return -1;
	}

	status = memalign(32, sizeof(fstats) );
	ret = ISFS_GetFileStats(fd, status);
	if (ret < 0)
	{
		printf("\nISFS_GetFileStats(fd) returned %d\n", ret);
		logfile("\nISFS_GetFileStats(fd) returned %d\n", ret);
		ISFS_Close(fd);
		fclose(file);
		free(status);
		return ret;
	}
	Con_ClearLine();

	printf("Dumping file %s, size = %uKB\n", source, (status->file_length / 1024)+1);
	logfile("Dumping file %s, size = %uKB\n", source, (status->file_length / 1024)+1);

    fs_size+=status->file_length;

	buffer = (u8 *)memalign(32, BLOCKSIZE);
	u32 restsize = status->file_length;
	while (restsize > 0)
	{
		if (restsize >= BLOCKSIZE)
		{
			size = BLOCKSIZE;
		} else
		{
			size = restsize;
		}
		ret = ISFS_Read(fd, buffer, size);
		if (ret < 0)
		{
			printf("\nISFS_Read(%d, %p, %d) returned %d\n", fd, buffer, size, ret);
			logfile("\nISFS_Read(%d, %p, %d) returned %d\n", fd, buffer, size, ret);
			ISFS_Close(fd);
			fclose(file);
			free(status);
			free(buffer);
			return ret;
		}
		ret = fwrite(buffer, 1, size, file);
		if(ret < 0) 
		{
			printf("\nfwrite error%d\n", ret);
			logfile("\nfwrite error%d\n", ret);
			ISFS_Close(fd);
			fclose(file);
			free(status);
			free(buffer);
			return ret;
		}
		restsize -= size;
	}
	ISFS_Close(fd);
	fclose(file);
	free(status);
	free(buffer);

	return 0;
}


			
void sort(u8 *a, int cnt)
{
	u8 tmp;
	int i, j, m;
	
	for(i = 0; i < cnt; i++)
	{
		m = i;
		for(j = i; j < cnt; j++)
			if(a[j] < a[m])
				m = j;
		tmp = a[i];
		a[i] = a[m];
		a[m] = tmp;
	}
}

u8 *get_ioslist(u32 *cnt)
{
	u64 *buf;
	s32 i, res;
	u32 tcnt = 0, icnt = 0;
	u8 *ioses = NULL;
	
	//Get stored IOS versions.
	res = ES_GetNumTitles(&tcnt);
	if(res < 0)
	{
		printf("ES_GetNumTitles: Error! (result = %d)\n", res);
		sleep(5);
		//logfile("ES_GetNumTitles: Error! (result = %d)\n", res);
		Reboot();
	}
	buf = memalign(32, sizeof(u64) * tcnt);
	res = ES_GetTitles(buf, tcnt);
	if(res < 0)
	{
		printf("ES_GetTitles: Error! (result = %d)\n", res);
		sleep(5);
		//logfile("ES_GetTitles: Error! (result = %d)\n", res);
		Reboot();
	}
	//Ugly.
	for(i = 0; i < tcnt; i++)
	{
		if(*((u32 *)(&(buf[i]))) == 1 && (u32)buf[i] > 2 && (u32)buf[i] < 0x100)
		{
			icnt++;
			ioses = (u8 *)realloc(ioses, sizeof(u8) * icnt);
			ioses[icnt - 1] = (u8)buf[i];
			//logfile("Found IOS %3d\n", ioses[icnt - 1]);
		}
	}
	free(buf);
	sort(ioses, icnt);
	
	*cnt = icnt;
	return ioses;
}


int isdir(char *path)
{
	s32 res;
	u32 num = 0;

	res = ISFS_ReadDir(path, NULL, &num);
	if(res < 0)
		return 0;
		
	return 1;
}

void get_attribs(char *path, u32 *ownerID, u16 *groupID, u8 *ownerperm, u8 *groupperm, u8 *otherperm)
{
	s32 res;
	u8 attributes;
	res = ISFS_GetAttr(path, ownerID, groupID, &attributes, ownerperm, groupperm, otherperm);
	if(res != ISFS_OK)
	printf("Error getting attributes of %s! (result = %d)\n", path, res);
	logfile("Error getting attributes of %s! (result = %d)\n", path, res);
}

s32 __FileCmp(const void *a, const void *b)
{
	dirent_t *hdr1 = (dirent_t *)a;
	dirent_t *hdr2 = (dirent_t *)b;
	
	if (hdr1->type == hdr2->type)
	{
		return strcmp(hdr1->name, hdr2->name);
	} else
	{
		if (hdr1->type == DIRENT_T_DIR)
		{
			return -1;
		} else
		{
			return 1;
		}
	}
}
void SaveDeviceCert()
{
	u8 *buffer = memalign(32,0x180);
	if(buffer == NULL)
	{
		logfile("ERROR! buffer for devicecert was NULL\n");
		printf("ERROR! buffer for devicecert was NULL\n");
		sleep(2);
		exit(0);
	}	else
	{
		logfile("Buffer was correctly allocated\n");
	}	
	//sleep(1);
	ES_GetDeviceCert(buffer);
	//sleep(1);
	//logfile("DeviceCert hexdump: \n\n");
	//hexdump_log(buffer, 0x180);
	//logfile("\n\n");
	FILE *f;
	char buf[256];
	sprintf(buf, "%ssys", device);
	free(buffer);
}	
void getdir(char *path, dirent_t **ent, int *cnt)
{
	s32 res;
	u32 num = 0;

	int i, j, k;

	//Get the entry count.
	res = ISFS_ReadDir(path, NULL, &num);
	if(res != ISFS_OK)
	{
		printf("Error: could not get dir entry count! (result: %d)\n", res);
		logfile("Error: could not get dir entry count! (result: %d)\n", res);
		return;
	}

	//Allocate aligned buffer.
	char *nbuf = (char *)memalign(32, (ISFS_MAXPATH + 1) * num);
	char ebuf[ISFS_MAXPATH + 1];
	char pbuf[ISFS_MAXPATH + 1];

	if(nbuf == NULL)
	{
		printf("Error: could not allocate buffer for name list!\n");
		logfile("Error: could not allocate buffer for name list!\n");
		return;
	}

	//Get the name list.
	res = ISFS_ReadDir(path, nbuf, &num);
	if(res != ISFS_OK)
	{
		printf("Error: could not get name list! (result: %d)\n", res);
		logfile("Error: could not get name list! (result: %d)\n", res);
		return;
	}
	
	//Set entry cnt.
	*cnt = num;

	if (*ent)
	{
		free(*ent);
	}
	*ent = malloc(sizeof(dirent_t) * num);

	u32 ownerID = 0;
	u16 groupID = 0;
	u8 attributes = 0;
	u8 ownerperm = 0;
	u8 groupperm = 0;
	u8 otherperm = 0;
	//char *hmm = NULL;
	//int place = 0;
	//logfile("Searching for unsupported characters ...\n");
	//Split up the name list.
	for(i = 0, k = 0; i < num; i++)
	{
		//The names are seperated by zeroes.
		for(j = 0; nbuf[k] != 0; j++, k++)
			ebuf[j] = nbuf[k];
		ebuf[j] = 0;
		k++;
		/*
		hmm = strchr(ebuf, ':');

		if(hmm != NULL)
		{
			logfile("Found character that is not supported on FAT, changing ':' to '_'\n");
			place = strlen(ebuf)-strlen(hmm);
			logfile("place = %d\n", place);
			logfile("strncpy(hmm, '_', 1);\n");
			//ebuf[place] = '_';
			strncpy(hmm, "_", 1);
			logfile("new ebuf = %s\n", ebuf);
		}	*/
		//Fill in the dir name.
		strcpy((*ent)[i].name, ebuf);
		//Build full path.
		if(strcmp(path, "/") != 0)
			sprintf(pbuf, "%s/%s", path, ebuf);
		else
			sprintf(pbuf, "/%s", ebuf);
		//Dir or file?
		(*ent)[i].type = ((isdir(pbuf) == 1) ? DIRENT_T_DIR : DIRENT_T_FILE);

		res = ISFS_GetAttr(pbuf, &ownerID, &groupID, &attributes, &ownerperm, &groupperm, &otherperm);
		if(res != ISFS_OK)
		{
			ownerID = 0;
			groupID = 0;
			attributes = 0;
			ownerperm = 0;
			groupperm = 0;
			otherperm = 0;
		}
		(*ent)[i].ownerID = ownerID;
		(*ent)[i].groupID = groupID;
		(*ent)[i].attributes = attributes;
		(*ent)[i].ownerperm = ownerperm;
		(*ent)[i].groupperm = groupperm;
		(*ent)[i].otherperm = otherperm;
	}
	qsort(*ent, *cnt, sizeof(dirent_t), __FileCmp);
	
	free(nbuf);
//	free(hmm);
}

void set_highlight(bool highlight)
{
	if (highlight)
	{
		printf("\x1b[%u;%um", 47, false);
		printf("\x1b[%u;%um", 30, false);
	} else
	{
		printf("\x1b[%u;%um", 37, false);
		printf("\x1b[%u;%um", 40, false);
	}
}


bool dumpfolder(char source[1024], char destination[1024], bool full, bool test_mode)
{
	//printf("Entering folder: %s\n", source);
	//if(SDHC_IsInserted() == false) { printf("ERROR, SD Card removed\n"); sleep(5); exit(0); }
	
	s32 tcnt;
	int ret;
	int i;
	char path[1024];
	char path2[1024];
	dirent_t *dir = NULL;
	u32 pressed;
	u32 pressedGC;

	getdir(source, &dir, &tcnt);

	if (strlen(source) == 1)
	{
		sprintf(path, "%s", destination);
	} else
	{
		sprintf(path, "%s%s", destination, source);
	}

	ret = (u32)opendir(path);
	if (ret == 0)
	{
		logfile("Dir %s did not exist, making it ...\n", path);
		ret = mkdir(path, 0777);
		if (ret < 0)
		{
			printf("Error making directory %d...\n", ret);
			logfile("Error making directory %d...\n", ret);
			free(dir);
			return false;
		}
	}
	
	for(i = 0; i < tcnt; i++) 
	{				
		if (strlen(source) == 1)
		{
			sprintf(path, "%s%s", source, dir[i].name);
		} else
		{
			sprintf(path, "%s/%s", source, dir[i].name);
		}
		
		if(dir[i].type == DIRENT_T_FILE) 
		{
			sprintf(path2, "%s%s", destination, path);

			//printf("Dumping file: %s\n", path);
			//printf("To: %s\n", path2);

			//sleep(5);
			ret = dumpfile(path, path2);
			if (ret < 0)
			{
			
				sleep(1);
				printf("Dumping complete now press A");
				if (strlen(source) == 1)
				{
					free(dir);
					return false;
				}
			}
		} else
		{
			if(dir[i].type == DIRENT_T_DIR) 
			{
				if(full == false && test_mode == false)
				{
					if(ignore_folder(path) != 0)
					{
						if (!dumpfolder(path, destination, full, test_mode))
						{
							free(dir);
							return false;
						}
					}	else
					{
						printf("          ", dir[i].name);
						logfile("Ignoring folder %s\n", dir[i].name);
					}
				}
				if(full == false && test_mode == true)
				{
					if(ignore_folder_test(path) != 0)
					{
						if (!dumpfolder(path, destination, full, test_mode))
						{
							free(dir);
							return false;
						}
					}	else
					{
						printf("        ", dir[i].name);
						logfile("Ignoring folder %s\n", dir[i].name);
					}
				}	
				if(full == true && test_mode == false)
				{
					if (!dumpfolder(path, destination, full, test_mode))
					{
						free(dir);
						return false;
					}
				}	
			}	
		}
	}
	free(dir);
	//printf("Dumping folder %s complete\n\n", source);
	return true;
}	

int ios_selectionmenu(int default_ios)
{
	s32 pressed;
	s32 pressedGC;
	int selection = 0;
	int buf = 1;
	u32 ioscount;
	u8 *list = get_ioslist(&ioscount);
	
	int i;
	for (i=0;i<ioscount;i++)
	{
		// Default to default_ios if found, else the loaded IOS
		if (list[i] == default_ios)
		{
			selection = i;
			break;
		}
		if (list[i] == IOS_GetVersion())
		{
			selection = i;
		}
	}	
	resetscreen();
	while (true)
	{
		//printf("\x1B[%d;%dH",0,0);	// move console cursor to x/y
		if(selection != buf)
		{
			buf = selection;
			resetscreen();	
			printf("Currently using IOS%u (Rev %u)\n", IOS_GetVersion(), IOS_GetRevision());
			set_highlight(true);
			printf("Select the IOS to load for Simple FS Dumper 0.42: %3d", list[selection]);
			set_highlight(false);
			printf("\nYou should use the latest cIOS 249\n");
		}
		
	
		WPAD_ScanPads();
		pressed = WPAD_ButtonsDown(0);
		PAD_ScanPads();
		pressedGC = PAD_ButtonsDown(0);
		if (pressedGC)
		{
			// Without waiting you can't select anything
			usleep (20000);
		}

		if (pressed == WPAD_BUTTON_LEFT || pressedGC == PAD_BUTTON_LEFT)
		{	
			if (selection > 0)
			{
				selection--;
			} else
			{
				selection = ioscount - 1;
			}
		}
		if (pressed == WPAD_BUTTON_RIGHT || pressedGC == PAD_BUTTON_RIGHT)
		{
			if (selection < ioscount -1	)
			{
				selection++;
			} else
			{
				selection = 0;
			}
		}
		if (pressed == WPAD_BUTTON_A || pressedGC == PAD_BUTTON_A) break;
		if (pressed == WPAD_BUTTON_B || pressedGC == PAD_BUTTON_B)
		{
			return 0;
		}
		
	}
	return list[selection];
}
/*
#define MINIMAL_DUMP 1
#define COMPATIBILITY_TEST_DUMP 2
#define FULL_DUMP 3
*/
int dump_selectionmenu()
{
	s32 pressed;
	s32 pressedGC;
	int selection = 1;
	int buf = 1;
	char *list[3] = {"Sync Dump (To sync an existing nand dump)", "EXIT", "EXIT"};
	{
			buf = selection;
			resetscreen();
			printf("What do you want to do?\n");
			set_highlight(true);
			printf("%s", list[selection]);
			set_highlight(false);
		}
	
	while (true)
	{
		//printf("\x1B[%d;%dH",0,0);	// move console cursor to x/y
		if(selection != buf)
		{
			buf = selection;
			resetscreen();
			printf("What do you want to do?\n");
			set_highlight(true);
			printf("%s", list[selection]);
			set_highlight(false);
		}
	
		WPAD_ScanPads();
		pressed = WPAD_ButtonsDown(0);
		PAD_ScanPads();
		pressedGC = PAD_ButtonsDown(0);
		if (pressedGC)
		{
			// Without waiting you can't select anything
			usleep (20000);
		}

		if (pressed == WPAD_BUTTON_LEFT || pressedGC == PAD_BUTTON_LEFT)
		{	
			if (selection > 0)
			{
				selection--;
			} else
			{
				selection = 2;
			}
		}
		if (pressed == WPAD_BUTTON_RIGHT || pressedGC == PAD_BUTTON_RIGHT)
		{
			if (selection < 2)
			{
				selection++;
			} else
			{
				selection = 0;
			}
		}
		if (pressed == WPAD_BUTTON_A || pressedGC == PAD_BUTTON_A) { break; }

		
	}
	return selection;
}

void device_selectionmenu(char *in)
{
	s32 pressed;
	s32 pressedGC;
	int selection = 2;
	int buf = 1;
	char *list[3] = {"SD", "USB 2.0 (Fast, use ONLY when a cIOS is loaded !)", "USB 1.1 (Slow)"};
	char *reallist[3] = {"sd:/", "usb2:/", "usb1:/"};
	resetscreen();
	
	while (true)
	{
		//printf("\x1B[%d;%dH",0,0);	// move console cursor to x/y
		if(selection != buf)
		{
			buf = selection;
			resetscreen();
			printf("Select the device:\n");
			set_highlight(true);
			printf("%s", list[selection]);
			set_highlight(false);
		}
	
		WPAD_ScanPads();
		pressed = WPAD_ButtonsDown(0);
		PAD_ScanPads();
		pressedGC = PAD_ButtonsDown(0);
		if (pressedGC)
		{
			// Without waiting you can't select anything
			usleep (20000);
		}

		if (pressed == WPAD_BUTTON_LEFT || pressedGC == PAD_BUTTON_LEFT)
		{	
			if (selection > 0)
			{
				selection--;
			} else
			{
				selection = 2;
			}
		}
		if (pressed == WPAD_BUTTON_RIGHT || pressedGC == PAD_BUTTON_RIGHT)
		{
			if (selection < 2)
			{
				selection++;
			} else
			{
				selection = 0;
			}
		}
		if (selection = 2) { break; }

		
	}
	sprintf(in, "%s", reallist[selection]);
}

int main(int argc, char **argv)
{
	Power_Flag = false;
	Reset_Flag = false;
	SYS_SetPowerCallback (power_cb);
    SYS_SetResetCallback (reset_cb);

	s32 ret;
	char dumpmodechar[256];

	char cpath[ISFS_MAXPATH + 1];	
	dirent_t* ent = NULL;
	int lcnt;
	device = memalign(32, 10);

	sys_init();

	PAD_Init();
	WPAD_Init();
	WPAD_SetDataFormat(WPAD_CHAN_0, WPAD_FMT_BTNS_ACC_IR);					

	int ios;
	ios = 58;
	if (ios != 58)
	{
		WPAD_Shutdown();
		PAD_Init();
		WPAD_Init();
		WPAD_SetDataFormat(WPAD_CHAN_0, WPAD_FMT_BTNS_ACC_IR);					
	}
	
	sprintf(cpath, "/");

	
	resetscreen();
	device_selectionmenu(device);
	//logfile("device = %s\n", device);
	resetscreen();
	sleep(1);
	if(strcmp(device, "sd:/") == 0)
	{
		__io_wiisd.startup();
		fatMountSimple("sd", &__io_wiisd);
		reset_log();
		logfile("Using SD\n");
	}	
	if(strcmp(device, "usb1:/") == 0)
	{
		__io_usbstorage.startup();
		fatMountSimple("usb1", &__io_usbstorage);
		reset_log();
		logfile("Using USB1\n");
	}	
	if(strcmp(device, "usb2:/") == 0)
	{
		__io_wiiums.startup();
		fatMountSimple("usb2", &__io_wiiums);
		reset_log();
		logfile("Using USB2\n");
	}	
	//__io_wiisd.startup();
	//fatMountSimple("sd", &__io_wiisd);
	sleep(1);
	
	ret = ISFS_Initialize();
	if (ret < 0) 
	{
		printf("Error: ISFS_Initialize returned %d\n", ret);
		logfile("Error: ISFS_Initialize returned %d\n", ret);
		sleep(5);
		Reboot();
	}


	resetscreen();
	int dumpmode = dump_selectionmenu();
	logfile("dumpmode = %d\n", dumpmode);
	resetscreen();
	switch(dumpmode)
	{
		case 0:
			sprintf(dumpmodechar, "Sync Dump");
			logfile("Dump Mode : %s\n", dumpmodechar);
			break;
		case 1:
	    exit(0);
			break;
		case 2:
		exit(0);
			break;	
		default:
			Reboot();
			break;
	}	
	printf("Nandsync for wii 0.9 by Magsor, Largely based on Simple FS Dumper by Nicksasa\n\n"); 

	printf("Also thanks to Wiipower for original code and the whole gbatemp community.\n\n");
	
	printf("Mode: %s | Device : %s\n", dumpmodechar, device);
	logfile("Mode: %s | Device : %s\n", dumpmodechar, device);

	

	
	getdir("/", &ent, &lcnt);
	
	//char devbuf[256];
	//sprintf(devbuf, "%s
	
		
	switch(dumpmode)
	{
		case 0:
			logfile("Going to save the device cert ...\n");
				if (dumpfolder("/", device, false, false))
			{
				printf("Your dumped fs was %u MB\n", (fs_size/1024)/1024);
				logfile("Your dumped fs was %u MB\n", (fs_size/1024)/1024);
				printf("\nDumping complete. Press any button to continue...\n");
				logfile("\nDumping complete. Press any button to continue...\n");
			}	
			break;
		case 1:
			logfile("Going to save the device cert ...\n");
				if (dumpfolder("/", device, false, true))
			{
				printf("Your dumped fs was %u MB\n", (fs_size/1024)/1024);
				logfile("Your dumped fs was %u MB\n", (fs_size/1024)/1024);
				printf("\nDumping complete. Press any button to continue...\n");
				logfile("\nDumping complete. Press any button to continue...\n");
			}	
			break;	
		case 2:
			logfile("Going to save the device cert ...\n");
					if (dumpfolder("/", device, true, false))
			{
				printf("Your dumped fs was %u MB\n", (fs_size/1024)/1024);
				logfile("Your dumped fs was %u MB\n", (fs_size/1024)/1024);
				printf("\nDumping complete. Press any button to continue...\n");
				logfile("\nDumping complete. Press any button to continue...\n");
			}	
			break;	
		default:
			exit(0);
	}		
	waitforbuttonpress(NULL, NULL);
	if(strcmp(device, "sd:/") == 0)
	{
		fatUnmount("sd");
		__io_wiisd.shutdown();
	}	
	if(strcmp(device, "usb1:/") == 0)
	{
		fatUnmount("usb1");
		__io_usbstorage.shutdown();
	}	
	if(strcmp(device, "usb2:/") == 0)
	{
		fatUnmount("usb2");
		__io_wiiums.shutdown();
	}	
	ISFS_Deinitialize();
	exit(0);
		
	free(device);
	
			
	
}
